Package: EpiTxDb
Type: Package
Title: Storing and accessing epitranscriptomic information using the
        AnnotationDbi interface
Version: 1.23.0
Date: 2025-09-23
Authors@R: person("Felix G.M.", 
                  "Ernst", 
                  email = "felix.gm.ernst@outlook.com", 
                  role = c("aut", "cre"),
                  comment = c(ORCID = "0000-0001-5064-0928"))
Description: 
    EpiTxDb facilitates the storage of epitranscriptomic information. More 
    specifically, it can keep track of modification identity, position, the
    enzyme for introducing it on the RNA, a specifier which determines the
    position on the RNA to be modified and the literature references each
    modification is associated with.
License: Artistic-2.0
Encoding: UTF-8
LazyData: false
biocViews: Software, Epitranscriptomics
Depends: R (>= 4.0), AnnotationDbi, Modstrings
Imports: methods, utils, httr, xml2, curl, rex, GenomicFeatures,
        txdbmaker, GenomicRanges, Seqinfo, BiocGenerics, BiocFileCache,
        S4Vectors, IRanges, RSQLite, DBI, Biostrings, tRNAdbImport
Suggests: BiocStyle, knitr, rmarkdown, testthat, httptest,
        AnnotationHub, ensembldb, ggplot2, EpiTxDb.Hs.hg38,
        BSgenome.Hsapiens.UCSC.hg38, BSgenome.Scerevisiae.UCSC.sacCer3,
        TxDb.Hsapiens.UCSC.hg38.knownGene
Collate: 'AllGenerics.R' 'EpiTxDb-SELECT-helpers.R' 'EpiTxDb-schema.R'
        'EpiTxDb.R' 'EpiTxDb-class.R' 'makeEpiTxDb.R'
        'makeEpiTxDbFromGRanges.R' 'shiftGenomicToTranscript.R'
        'makeEpiTxDbFromRMBase.R' 'makeEpiTxDbFromtRNAdb.R'
        'modifications.R' 'modificationsBy.R' 'ranges-helpers.R'
        'select-methods.R'
RoxygenNote: 7.3.3
BugReports: https://github.com/FelixErnst/EpiTxDb/issues
URL: https://github.com/FelixErnst/EpiTxDb
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/EpiTxDb
git_branch: devel
git_last_commit: 23585ca
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 22:30:03 UTC; biocbuild
Author: Felix G.M. Ernst [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5064-0928>)
Maintainer: Felix G.M. Ernst <felix.gm.ernst@outlook.com>
