# Generated by roxygen2: do not edit by hand

export(EPITXDB_RMBASE_URL)
export(downloadRMBaseFiles)
export(getRMBaseDataAsGRanges)
export(gettRNAdbDataAsGRanges)
export(listAvailableGenomesFromRMBase)
export(listAvailableModFromRMBase)
export(listAvailableOrganismsFromRMBase)
export(listAvailableOrganismsFromtRNAdb)
export(makeEpiTxDb)
export(makeEpiTxDbFromGRanges)
export(makeEpiTxDbFromRMBase)
export(makeEpiTxDbFromRMBaseFiles)
export(makeEpiTxDbFromtRNAdb)
exportClasses(EpiTxDb)
exportMethods(as.integer)
exportMethods(as.list)
exportMethods(columns)
exportMethods(keys)
exportMethods(keytypes)
exportMethods(modifications)
exportMethods(modificationsBy)
exportMethods(modifiedSeqsByTranscript)
exportMethods(organism)
exportMethods(positionSequence)
exportMethods(rescale)
exportMethods(select)
exportMethods(seqinfo)
exportMethods(seqlevels)
exportMethods(shiftGenomicToTranscript)
exportMethods(shiftTranscriptToGenomic)
import(AnnotationDbi)
import(BiocGenerics)
import(IRanges)
import(Modstrings)
import(RSQLite)
import(S4Vectors)
import(Seqinfo)
import(methods)
import(rex)
import(txdbmaker)
importClassesFrom(IRanges,PartitioningByEnd)
importClassesFrom(IRanges,PartitioningByWidth)
importFrom(BiocFileCache,BiocFileCache)
importFrom(BiocFileCache,bfcadd)
importFrom(BiocFileCache,bfcquery)
importFrom(BiocGenerics,organism)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,subseq)
importFrom(Biostrings,vwhichPDict)
importFrom(DBI,dbListFields)
importFrom(DBI,dbListTables)
importFrom(GenomicFeatures,transcripts)
importFrom(GenomicRanges,findOverlaps)
importFrom(IRanges,PartitioningByEnd)
importFrom(IRanges,PartitioningByWidth)
importFrom(S4Vectors,findMatches)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlevels)
importFrom(curl,curl)
importFrom(httr,POST)
importFrom(httr,content)
importFrom(httr,modify_url)
importFrom(tRNAdbImport,import.tRNAdb)
importFrom(utils,packageDescription)
importFrom(utils,read.delim)
importFrom(xml2,xml_find_all)
