% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EpiTxDb.R
\docType{data}
\name{EpiTxDb-data}
\alias{EpiTxDb-data}
\alias{rmbase_data}
\title{EpiTxDb internal data}
\format{
data.frame
}
\usage{
data(rmbase_data)
}
\description{
EpiTxDb internal data
}
\keyword{datasets}
